Package: mipfp
Type: Package
Title: Multidimensional Iterative Proportional Fitting and Alternative
        Models
Version: 3.2.3
Date: 2026-08-22
Authors@R: c(
    person("Johan", "Barthelemy", role = c("aut", "cre"), email = "johan@uow.edu.au"), 
    person("Thomas", "Suesse", role = "aut"), 
    person("Mohammad", "Namazi-Rad", role = "ctb"))
Author: Johan Barthelemy [aut, cre],
  Thomas Suesse [aut],
  Mohammad Namazi-Rad [ctb]
Maintainer: Johan Barthelemy <johan@uow.edu.au>
Description: An implementation of the iterative proportional fitting (IPFP), 
    maximum likelihood, minimum chi-square and weighted least squares procedures
    for updating a N-dimensional array with respect to given target marginal 
    distributions (which, in turn can be multidimensional). The package also
    provides an application of the IPFP to simulate multivariate Bernoulli
    distributions.
LazyData: yes
Depends: cmm, Rsolnp, numDeriv, R(>= 2.10.0)
License: GPL-2
URL: https://github.com/jojo-/mipfp
NeedsCompilation: no
Packaged: 2026-08-22 05:43:47 UTC; johan
Repository: CRAN
Date/Publication: 2026-08-22 06:30:02 UTC
