exportPattern("^[[:alpha:]]+")
importFrom(cmm,MarginalMatrix)
importFrom(Rsolnp,solnp)
importFrom(numDeriv,jacobian)
importFrom(stats,qnorm,rmultinom)
importFrom(utils,tail)
importFrom("stats", "coef", "pchisq", "printCoefmat", "pt", "vcov")
S3method(coef, mipfp)
S3method(confint, mipfp)
S3method(error.margins, default)
S3method(error.margins, mipfp)
S3method(flat, default)
S3method(flat, array)
S3method(flat, matrix)
S3method(flat, table)
S3method(expand, default)
S3method(expand, table)
S3method(gof.estimates, default)
S3method(gof.estimates, mipfp)
S3method(print, mipfp)
S3method(print, summary.mipfp)
S3method(summary, mipfp)
S3method(vcov, mipfp)
